/*
 Thermocouple.h
 Copyright (c) 2016, Dust Networks.  All rights reserved.
 
 Subject to the terms of the OCSDK license - https://github.com/dustcloud/onchipsdk/blob/master/software_license_on-chip_sdk.pdf
*/

#ifndef THERMOCOUPLE_H
#define THERMOCOUPLE_H

#ifndef TIME_DRV_H
#include "dn_time.h"
#endif


// SDK includes
#include "dn_common.h"
#include "dn_system.h"
#include "dn_spi.h"
#include "dn_i2c.h"
#include "dn_gpio.h"

// project includes
#include "app_task_cfg.h"
#include "LTC2983_configuration_constants.h"
#include "LTC2983_support_functions.h"

// C includes
#include "string.h"
#include "stdio.h"

//=========================== definitions =====================================
#define MAX_RX_PAYLOAD          80    // bytes
#define MAX_TX_PAYLOAD          90    // bytes

// device opened/closed
#define OPEN                    1
#define CLOSED                  0

//===== I2C
#define I2C_SLAVE_ADDR          0xC0
#define PERSONALITY_ADDR_EE     0x50  // 0xA0 in schematic
#define PERSONALITY_OFFSET_EE   0x00  // Addresses A0 and A1 are settable by Jumper and default to 0
#define I2C_MAX_BUFFER_SIZE     80

//===== SPI
#define SPI_MAX_BUFFER_SIZE     8

// transfers
#define READ                    1
#define WRITE                   0

//===== pins
#define BUS_SELECT              DN_GPIO_PIN_13_DEV_ID    // SPIM_SS_1n
#define PIN_LOW                 0x00
#define PIN_HIGH                0x01
#define BUS_I2C                 0x01                    
#define BUS_SPI                 0x00
#define ENABLE                  1

// command TLV

#define PERSONALITY_LEN         50   // 50-byte personality string
#define BUS_SW_LEN              1
#define CONFIG_I2C_LEN          8
#define CONFIG_SPI_LEN          14
#define RESPONSE_LEN            2
#define CMD_HDR_LEN             4

#define CH_1                    1
#define CH_2                    2
#define MEM_READ                0x03
#define MEM_WRITE               0x02
#define STATUS_REG              0x0000
#define CONFIG_REG              0x0F0
#define MUX_REG                 0x0FF

#define NOTIFICATION     5
#define RAW_SAMPLE       0
#define SAMPLE_TAG       0xFF
#define TEMP_MEAS        0x5   //temperature
#define BITS_16          0x10

#define SAMPLE_RATE      0x2710 //10000 ms

#define UNACKNOWLEDGED   0x00
#define REQUEST          0x00
#define SYNC             0x04
#define NOSYNC           0x00
#define SESSIONID        0x00

//=========================== message formats ================================

// All commands are app-layer acknowledged
typedef enum {
   
   DC2399_READ_PERSONALITY = 0,   // read the personality EEPROM
   DC2399_BUS_SWITCH,             // switch bus between SPI/I2C
   DC2399_CONFIG_I2C,             // configure the I2C interface
   DC2399_CONFIG_SPI,             // configure the SPI interface
   DC2399_RW_I2C,                 // read/write I2C command
   DC2399_RW_SPI,                 // read/write SPI command
   DC2399_DATA,                   // return other data
   DC2399_RESPONSE,               // response code
} command_types_t;

PACKED_START

typedef struct{
   INT8U      control;
   INT8U      id;
}oap_header_t;

typedef struct{   
   INT8U      tag;
   INT8U      length;
   INT8U      value;
}oap_channel_t;

typedef struct{
   INT64S     sec;
   INT32S     usec;   
}packet_utc_t;      // IAR doesn't pack dn_utc_time_t into the oap_temp_notification_t properly

typedef struct{   
   oap_header_t    header;
   INT8U           command;
   INT8U           type;
   oap_channel_t   channel;
   packet_utc_t    timestamp;
   INT32U          rate; //ms
   INT8U           num_samples;
   INT8U           sample_size; //bits
   INT16S          temperature;
}oap_temp_notification_t;

// Personality EEPROM read from the target
typedef struct{
   char         personality[50];
} personality_t;

typedef struct{
   INT8U        command;
   INT8S        rc; 
} response_t;

// command to switch the bus
typedef struct{
   INT8U        bus;    // one of BUS_I2C or BUS_SPI 
} bus_t;

typedef struct{  
   INT8U                        frequency;               //I2C clock frequency (for open args)
   dn_ioctl_i2c_transfer_t      config;
}i2c_config_t;

typedef struct{  
   dn_ioctl_spi_transfer_t      config;
}spi_config_t;

typedef struct{
   INT8U                        direction;                        // tells which buffer is being passed
   INT16U                       transactionLen;                   // how many bytes to read or write
   INT16U                       numSamples;                       // how many times to repeat  
   INT8U                        writeBuffer[MAX_RX_PAYLOAD];   
}payload_t;


typedef struct{
   INT16U                       magic;
   INT8U                        tag;                       
   INT8U                        length;
   union {                                                      // value   
      command_types_t           command;                        //
      personality_t             personality;                    // postage stamp personality string
      bus_t                     busSelect;                      // i2c or spi bus for target
      spi_config_t              spiConfig;                      // rxData and TxData are passed as NULL
      i2c_config_t              i2cConfig;                      // writeBuf and readBuf are passed as NULL
      response_t                response;                       // acknowledgement that a command was received  
      payload_t                 payload;                        // raw payload (US or DS)
   } command;
} OTA_command_t;

typedef struct{
   INT8U    command;
   INT16U   address; 
   INT8U    data[4];
} command_t;

PACKED_STOP

typedef struct {
   dn_ioctl_i2c_transfer_t      i2cTransfer;
   dn_ioctl_spi_transfer_t      spiTransfer;
   OS_STK                       appTaskStack[TASK_APP_DC2399_STK_SIZE];
   INT8U                        i2cReadBuffer[I2C_MAX_BUFFER_SIZE];
   INT8U                        i2cWriteBuffer[I2C_MAX_BUFFER_SIZE];
   INT8U                        spiTxBuffer[SPI_MAX_BUFFER_SIZE];
   INT8U                        spiRxBuffer[SPI_MAX_BUFFER_SIZE];
   INT8U                        i2cReadLen;
   INT8U                        i2cWriteLen;
   INT8U                        i2cSlaveAddr;
   INT8U                        i2cFrequency;
   dn_spi_open_args_t           spiOpenArgs;
   INT8U                        fI2cOpen;
   INT8U                        fSpiOpen;
   INT8U                        bus;
   OS_EVENT*                    joinedSem;
   OS_EVENT*                    rxReady;                    ///< posted when an rx packet is ready for consumption
   INT8U                        rxPayload[MAX_RX_PAYLOAD];  ///< received payload
   INT8U                        rxPayloadLen;               ///< number of bytes in the received payload
   personality_t                personality;
   spi_config_t                 spiConfig;
   i2c_config_t                 i2cConfig;
   command_t                    command;
   INT8U                        cmdLen;
   oap_temp_notification_t      oap_temp;
} dc2399_app_vars_t;

#endif