/*
 Thermocouple_app.c
 Copyright (c) 2016, Dust Networks.  All rights reserved.
 
 Subject to the terms of the OCSDK license - https://github.com/dustcloud/onchipsdk/blob/master/software_license_on-chip_sdk.pdf
*/

/* =================================================================================

The DC2399 board sits between a DC9003 and a target board. It allows a host
application to wirelessly read the personality EEPROM from the target, switch to 
an I2C or SPI BUS, then send read/write commands to the target chip.

This is intended to provide similar function to the QuickEval DC590B board.

To access the personality "postage stamp" EEPROM:
* Write the address (1-byte write) 0x00
* Read each byte sequentially

=================================================================================*/

// SDK includes
#include "dn_common.h"
#include "dn_system.h"
#include "dn_spi.h"
#include "dn_i2c.h"
#include "dn_gpio.h"
#include "dn_exe_hdr.h"
#include "cli_task.h"
#include "loc_task.h"
#include "well_known_ports.h"
#include "dn_time.h"
#include "Ver.h"

// project includes
#include "app_task_cfg.h"
#include "LTC2983_configuration_constants.h"
#include "LTC2983_support_functions.h"
#include "Thermocouple.h"

// C includes
#include "string.h"
#include "stdio.h"

//=========================== variables =======================================
dc2399_app_vars_t     app_v;

//=========================== prototypes ======================================

// TASKS
static void appTask(void* unused);

// GPIO
dn_error_t gpioSetMode(INT8U pin, INT8U mode, INT8U pull, INT8U initLevel);
dn_error_t gpioWrite(INT8U pin,INT8U value);

//SPI
dn_error_t spiOpen();
dn_error_t spiRead();
dn_error_t spiWrite();
dn_error_t spiClose();

//I2C
dn_error_t i2cOpen();
dn_error_t i2cRead();
dn_error_t i2cWrite();
dn_error_t i2cClose();

//=== Command Line Interface (CLI) handlers =======
dn_error_t cli_reset(INT8U* arg, INT32U len);
dn_error_t cli_read_EE(INT8U* arg, INT32U len);

// network commands
void send();

//=========================== const  ==============================================
// Note - help strings for user defined CLI commands don't work in stack 1.0.3.28
const dnm_ucli_cmdDef_t cliCmdDefs[] = {
  {&cli_reset,                      "reset",          "reset",       DN_CLI_ACCESS_LOGIN },
  {&cli_read_EE,                    "EE",             "EE",          DN_CLI_ACCESS_LOGIN },
  {NULL,                            NULL,             NULL,                             0},
};

/**
\brief This is the entry point in the application code.
*/
int p2_init(void) {
   dn_error_t              status;
   dn_error_t              dnErr;
   INT8U                   osErr;
  
   app_v.joinedSem = OSSemCreate(0);    // create a semaphore to indicate mote joined
   ASSERT(app_v.joinedSem!=NULL);

   cli_task_init(
      "DC2399A",                        // appName
      cliCmdDefs                        // cliCmds
   );
   
   loc_task_init(
      JOIN_YES,                         // fJoin
      NETID_NONE,                       // netId
      WKP_USER_2,                       // udpPort
      app_v.joinedSem,                  // joinedSem
      BANDWIDTH_NONE,                   // bandwidth
      NULL                              // serviceSem
   );
   
   // create the app task
   osErr  = OSTaskCreateExt(
      appTask,
      (void *)0,
      (OS_STK*)(&app_v.appTaskStack[TASK_APP_DC2399_STK_SIZE-1]),
      TASK_APP_DC2399_PRIORITY, 
      TASK_APP_DC2399_PRIORITY,
      (OS_STK*)app_v.appTaskStack,
      TASK_APP_DC2399_STK_SIZE,
      (void *)0,
      OS_TASK_OPT_STK_CHK | OS_TASK_OPT_STK_CLR
   );
   ASSERT(osErr==OS_ERR_NONE);
   OSTaskNameSet(TASK_APP_DC2399_PRIORITY, (INT8U*)TASK_APP_DC2399_NAME, &osErr);
   ASSERT(osErr==OS_ERR_NONE);
   
   return (DN_ERR_NONE);
}

//========== CLI handler functions : These get called when a command is entered on the CLI =========

//===== CLI command to reset the mote
// @param arg = a string of arguments passed to the CLI command
// @param len = the length of the argument string
//
// @return  returns an error if dnm_loc_resetCmd fails
dn_error_t cli_reset(INT8U* arg, INT32U len){
   INT8U      rc;
   
   dnm_ucli_printf("Resetting...\r\n\n");
   
   // send reset to stack
   dnm_loc_resetCmd(&rc);	
   
   return(rc);
}

//===== CLI command to reset the mote
// @param arg = a string of arguments passed to the CLI command
// @param len = the length of the argument string
//
// @return  returns an error if dnm_loc_resetCmd fails
dn_error_t cli_read_EE(INT8U* arg, INT32U len){
   dn_error_t    dnErr;
   INT8U         rc;
   int           i;
   INT8U         personalityStr;
   
   // set the slave address to the personality "postage stamp" EEPROM
   app_v.i2cSlaveAddr = PERSONALITY_ADDR_EE | PERSONALITY_OFFSET_EE;
   
   if(app_v.fI2cOpen == CLOSED){
       dnErr = i2cOpen();
       app_v.fI2cOpen = OPEN;
   }
   
   //fill the write buffer
   app_v.i2cWriteLen = 1;
   app_v.i2cWriteBuffer[0] = 0x00;
   dnErr = i2cWrite();
   
   //read the contents of the EEPROM and dump to screen
   app_v.i2cReadLen = 1; //I2C_MAX_BUFFER_SIZE;
   memset(&app_v.i2cReadBuffer, 0, sizeof(app_v.i2cReadBuffer));
   
   dnm_ucli_printf("Personality EEPROM:  ");
   for (i= 0; i< 0xFF; i++){
      dnErr = i2cRead();
      if (dnErr == DN_ERR_NONE){
         dnm_ucli_printf("%c", app_v.i2cReadBuffer[0]);
      }
      else {
         dnm_ucli_printf("..");
      }
   }
   dnm_ucli_printf("\r\n");
   
   return (DN_ERR_NONE);
}


//=========================== initialization ==================================

//=========================== I2C task ========================================

// ======== Received packet processor task
static void appTask(void* unused) {
   dn_error_t                   dnErr;
   INT8U                        osErr;
   INT8U                        i;
   INT8U                        rc;
   INT8U                        pkBuf[sizeof(loc_sendtoNW_t) + MAX_TX_PAYLOAD];
   loc_sendtoNW_t*              pkToSend;
   OTA_command_t                response;
   INT32U                       channel_assignment_data;
   oap_temp_notification_t      report;
   INT8U                        firstPacket = 1;
   INT8U                        report_ctr = 0;
   dn_time_asn_t                asn;
   dn_time_utc_t                utc;
   INT32U                       rawTemperature;
   INT16S                       temperature;
   
   // wait a bit
   OSTimeDly(1000);
   
   // open the SPI device 
   dnErr = dn_open(
      DN_SPI_DEV_ID,
      &app_v.spiOpenArgs,
      sizeof(app_v.spiOpenArgs)
   );
   ASSERT(dnErr == DN_ERR_NONE);
   
   //===== open the I2C device for reading the personality EEPROM
   if(app_v.fI2cOpen == CLOSED){
       dnErr = i2cOpen();
       app_v.fI2cOpen = OPEN;
   }
   
     // ===== open the BUS_SELECT pin
   dnErr = gpioSetMode(BUS_SELECT, DN_IOCTL_GPIO_CFG_OUTPUT, NULL, PIN_LOW);
   if (dnErr != DN_ERR_NONE){
      dnm_ucli_printf("bus pin open err=%d\r\n", dnErr);
   }
   
   dnErr = gpioWrite(BUS_SELECT, BUS_SPI);
   if (dnErr != DN_ERR_NONE){
      dnm_ucli_printf("bus pin write err=%d\r\n", dnErr);
   }
   
   // initialize spi communication parameters
   app_v.spiTransfer.txData             = app_v.spiTxBuffer;
   app_v.spiTransfer.rxData             = app_v.spiRxBuffer;
   app_v.spiTransfer.clockPhase         = DN_SPI_CPHA_0;//app_v.spiConfig.config.clockPhase;
   app_v.spiTransfer.clockPolarity      = DN_SPI_CPOL_0; //app_v.spiConfig.config.clockPolarity;
   app_v.spiTransfer.clockDivider       = DN_SPI_CLKDIV_16; //app_v.spiConfig.config.clockDivider;
   app_v.spiTransfer.startDelay         = 1;  //app_v.spiConfig.config.startDelay;
   app_v.spiTransfer.bitOrder           = DN_SPI_MSB_FIRST;// app_v.spiConfig.config.bitOrder;
   app_v.spiTransfer.slaveSelect        = DN_SPIM_SS_0n; //app_v.spiConfig.config.slaveSelect;
   app_v.spiTransfer.transactionLen     = 8; // app_v.spiConfig.config.transactionLen;
   app_v.spiTransfer.numSamples         = 1; // app_v.spiConfig.config.numSamples;
   
   //===== initialize packet variables
   pkToSend = (loc_sendtoNW_t*)pkBuf;
   pkToSend->locSendTo.socketId          = loc_getSocketId();
   pkToSend->locSendTo.destAddr          = DN_MGR_IPV6_MULTICAST_ADDR; // IPv6 address
   pkToSend->locSendTo.destPort          = WKP_USER_2;
   pkToSend->locSendTo.serviceType       = DN_API_SERVICE_TYPE_BW;   
   pkToSend->locSendTo.priority          = DN_API_PRIORITY_MED;   
   pkToSend->locSendTo.packetId          = 0xFFFF;
   
   // setup OAP packet
   app_v.oap_temp.command = NOTIFICATION;
   app_v.oap_temp.type = RAW_SAMPLE;
   app_v.oap_temp.channel.tag = SAMPLE_TAG;
   app_v.oap_temp.channel.length = 1;
   app_v.oap_temp.channel.value = TEMP_MEAS;
   app_v.oap_temp.rate = htons(SAMPLE_RATE); 
   app_v.oap_temp.num_samples = 1;
   app_v.oap_temp.sample_size = BITS_16;
   
   OSSemPend(app_v.joinedSem, 0, &osErr);
   ASSERT(osErr==OS_ERR_NONE);
     
   while(1) {  // infinite loop    
      
       // -- Set global config register
      rwByte(CONFIG_REG, MEM_WRITE, TEMP_UNIT__C | REJECTION__50_60_HZ);
       // -- Set delay between conversions
      rwByte(MUX_REG, MEM_WRITE, 0);
      
      //----- Channel 1: Assign Type K Thermocouple -----
      channel_assignment_data = SENSOR_TYPE__TYPE_K_THERMOCOUPLE | TC_COLD_JUNCTION_CH__2 | TC_SINGLE_ENDED |
                             TC_OPEN_CKT_DETECT__YES | TC_OPEN_CKT_DETECT_CURRENT__10UA;
      assignChannel(CH_1, channel_assignment_data);
      
      //----- Channel 2: Diode reference -----
      channel_assignment_data = SENSOR_TYPE__OFF_CHIP_DIODE | DIODE_SINGLE_ENDED | DIODE_NUM_READINGS__2 |
                             DIODE_AVERAGING_OFF | DIODE_CURRENT__20UA_80UA_160UA | (0x100C49 << DIODE_IDEALITY_FACTOR_LSB);
      assignChannel(CH_2, channel_assignment_data);
     
      convertChannel(CH_1); 
      OSTimeDly(500);  // wait 500 ms for conversion to complete instead of polling
      
      rwByte(STATUS_REG, MEM_READ, 0);
      
      rawTemperature = getRawResults(READ_CH_BASE, CH_1);   
      if(rawTemperature & 0x800000){  // mask off sign bit
        temperature = (INT16S)(rawTemperature & 0x7FFFFF)  >> 10;
        temperature *= (INT16S)-1;
      }   
      else {
         temperature = (INT16S)(rawTemperature >> 10);
      }
      
      dnm_ucli_printf("Channel=%d, Temperature Result = %d.%u\r\n", CH_1, temperature, rawTemperature & 0x3FF);
      app_v.oap_temp.temperature = htons(temperature);
      
      // Send an OAP temperature packet
      app_v.oap_temp.header.control = UNACKNOWLEDGED | REQUEST | NOSYNC;
      if (firstPacket) {
         app_v.oap_temp.header.control |= SYNC;
         firstPacket = 0;
      }
      
      app_v.oap_temp.header.id = (SESSIONID<<4) + report_ctr;
      report_ctr++;
      if (report_ctr > 15) {
         report_ctr=0;
      }
      
      //packet timestamp
      dn_getNetworkTime(&asn, &utc);
      app_v.oap_temp.timestamp.sec = htonl(utc.sec);
      app_v.oap_temp.timestamp.usec = htons(utc.usec);
            
      memcpy(&pkToSend->locSendTo.payload, &app_v.oap_temp, sizeof(app_v.oap_temp));
      
       // send the packet
      dnErr = dnm_loc_sendtoCmd(pkToSend, sizeof(oap_temp_notification_t), &rc);
      ASSERT (dnErr == DN_ERR_NONE);      
      
      OSTimeDly(10*SECOND);
      
   }
}

//========================== Helpers ==========================================

// ===== GPIO commands ============

// Setup GPIO pin
// @param pin       = which pin to set
// @param mode      = input or output
// @param pull      = direction of pullup for inputs
// @param initLevel = initial level for outputs
//
// @return returns RC from dn_ioctl (pin set) command
dn_error_t gpioSetMode(INT8U pin, INT8U mode, INT8U pull, INT8U initLevel){
   dn_error_t                   dnErr;
   dn_gpio_ioctl_cfg_in_t       gpioInCfg;
   dn_gpio_ioctl_cfg_out_t      gpioOutCfg; 
   
   dnErr = dn_open(
      pin,
      NULL,
      0
   );
   ASSERT(dnErr==DN_ERR_NONE);
   
   if (mode == DN_IOCTL_GPIO_CFG_OUTPUT){
     gpioOutCfg.initialLevel = initLevel;
     dnErr = dn_ioctl(
        pin,
        mode,
        &gpioOutCfg,
        sizeof(gpioOutCfg)
     );
     ASSERT(dnErr==DN_ERR_NONE);
   }
   
   if (mode == DN_IOCTL_GPIO_CFG_INPUT){
     gpioInCfg.pullMode = pull;
     dnErr = dn_ioctl(
        pin,
        mode,
        &gpioInCfg,
        sizeof(gpioInCfg)
     );
     ASSERT(dnErr==DN_ERR_NONE);
   }
  
   if ((mode != DN_IOCTL_GPIO_CFG_INPUT) && (mode != DN_IOCTL_GPIO_CFG_OUTPUT)){
      
      return (DN_ERR_INVALID);
   }
   else {
      
      return (DN_ERR_NONE);
   } 
}

// Write to GPIO output
// @param pin       = which pin to set
// @param value     = PIN_HIGH or PIN_LOW
//
dn_error_t gpioWrite(INT8U pin,INT8U value){
   dn_error_t  dnErr;
   
   dnErr = dn_write(
      pin,                  // device
      &value,               // buf
      sizeof(value)         // len
   );
   
   return (dnErr);  
}

//================== I2C

dn_error_t i2cOpen(){
   dn_error_t                   dnErr;
   dn_i2c_open_args_t           i2cOpenArgs;
   
   // open the I2C device
   i2cOpenArgs.frequency = DN_I2C_FREQ_184_KHZ;
   dnErr = dn_open(
      DN_I2C_DEV_ID,
      &i2cOpenArgs,
      sizeof(i2cOpenArgs)
   );
   
   return (dnErr);
}

dn_error_t i2cWrite(){
   dn_error_t  dnErr;
      
   // initialize I2C communication parameters
   app_v.i2cTransfer.slaveAddress    = app_v.i2cSlaveAddr;
   app_v.i2cTransfer.writeBuf        = app_v.i2cWriteBuffer;
   app_v.i2cTransfer.readBuf         = NULL;
   app_v.i2cTransfer.writeLen        = sizeof(app_v.i2cWriteLen);
   app_v.i2cTransfer.readLen         = 0;
   app_v.i2cTransfer.timeout         = 0xff;
      
   // initiate transaction
   dnErr = dn_ioctl(
   DN_I2C_DEV_ID,
   DN_IOCTL_I2C_TRANSFER,
   &app_v.i2cTransfer,
   sizeof(app_v.i2cTransfer)
   );
      
  return (dnErr);
}

dn_error_t i2cRead(){
   dn_error_t  dnErr;
      
   // initialize I2C communication parameters
   app_v.i2cTransfer.slaveAddress    = app_v.i2cSlaveAddr;
   app_v.i2cTransfer.writeBuf        = NULL;
   app_v.i2cTransfer.readBuf         = app_v.i2cReadBuffer;
   app_v.i2cTransfer.writeLen        = 0;
   app_v.i2cTransfer.readLen         = sizeof(app_v.i2cReadLen);
   app_v.i2cTransfer.timeout         = 0xff;
      
   // initiate transaction
   dnErr = dn_ioctl(
         DN_I2C_DEV_ID,
         DN_IOCTL_I2C_TRANSFER,
         &app_v.i2cTransfer,
         sizeof(app_v.i2cTransfer)
   ); 
   
   return (dnErr);
}

dn_error_t i2cClose(){
   dn_error_t   dnErr;
   
   // close the  device
   dnErr = dn_close(DN_I2C_DEV_ID);
   return (dnErr);
}

//================== SPI
dn_error_t spiOpen(){ 
   dn_error_t   dnErr;
   
   dnErr = dn_open(
      DN_SPI_DEV_ID,
      &app_v.spiOpenArgs,
      sizeof(app_v.spiOpenArgs)
   );
   
   return (dnErr);
}

dn_error_t spiRead(){
   
    return (DN_ERR_NONE);
}

dn_error_t spiWrite(){
   
    return (DN_ERR_NONE);
}

dn_error_t spiClose() {
   dn_error_t  dnErr;
   
    // close the  device
   dnErr = dn_close(DN_SPI_DEV_ID);
   return (dnErr);
}


//=============================================================================
//=========================== install a kernel header =========================
//=============================================================================

/**
A kernel header is a set of bytes prepended to the actual binary image of this
application. Thus header is needed for your application to start running.
*/

DN_CREATE_EXE_HDR(DN_VENDOR_ID_NOT_SET,
                  DN_APP_ID_NOT_SET,
                  VER_MAJOR,
                  VER_MINOR,
                  VER_PATCH,
                  VER_BUILD);
